// =================================================================
//
// CBP_3352_IProc_2.c 
// 
// World CBP_3352_IProc_2.wbt
// CBP 30-01-21
// Derived from experimental CBP_ePuck_IProc_2.c
//
// Applies segmentation, object extraction and subsampling to 7x7 as
// in _IProc_1.c
// Here we overlap with a pattern and calculate the overlap. This is
// managed interactively, by the user pressing some keys
// Routines from _IProc_1.c are now in CBP_IProc_2.c and overlap
// routines are in CBP_IProc_3.c
//
// ==================================================================

#include <stdio.h>
#include <webots/camera.h>
#include <webots/motor.h>
#include <webots/robot.h>
#include <webots/display.h>
#include <webots/distance_sensor.h>
#include <webots/keyboard.h>

#include <stdlib.h>

#define DEBUG false

#include "CBP_Helper_4.h"
#include "CBP_IProc_1.h"
#include "CBP_IProc_2.h"
#include "CBP_IProc_3.h"

#define TIME_STEP 32
#define MAX_SPEED 6.28
 
WbDeviceTag camera, left_motor, right_motor;
double omegaL, omegaR;

// Image processing stuff ----------------------------------------
WbDeviceTag camera;
// The camera image
const unsigned char *imageCamera;
int widthCamera, heightCamera;

// The segmented image
static unsigned char *imageSeg;
int widthImageSeg, heightImageSeg;
static WbDeviceTag displaySeg; 

// Image containing the extracted object
static unsigned char *imageObj;
int widthImageObj, heightImageObj;
static WbDeviceTag displayObj; 

// Image containing the subsampled image
static unsigned char *imageSubs;
int widthImageSubs, heightImageSubs;
static WbDeviceTag displaySubs;

//Pattern detection images
static unsigned char *pattern_image; 

// Grabs all devices needed and initializes them                                  
void init() {

  camera = wb_robot_get_device("camera");
  wb_camera_enable(camera, TIME_STEP);
  widthCamera = wb_camera_get_width(camera);
  heightCamera = wb_camera_get_height(camera);
  printf("Camera width: %d height: %d\n",widthCamera,heightCamera);

  /* get a handler to the motors and set target position to infinity (speed control). */
  left_motor = wb_robot_get_device("left wheel motor");
  right_motor = wb_robot_get_device("right wheel motor");
  wb_motor_set_position(left_motor, INFINITY);
  wb_motor_set_position(right_motor, INFINITY);
  wb_motor_set_velocity(left_motor, 0.0);
  wb_motor_set_velocity(right_motor, 0.0);

  // get the display device for segmented image
  displaySeg = wb_robot_get_device("display");
  widthImageSeg = wb_display_get_width(displaySeg); 
  heightImageSeg = wb_display_get_height(displaySeg);
 
   // get the display device for subsampled
  displaySubs = wb_robot_get_device("display3");
  widthImageSubs = wb_display_get_width(displaySeg); 
  heightImageSubs = wb_display_get_height(displaySeg);
  
  wb_keyboard_enable(TIME_STEP);
 
}

// Keyboard stuff ----------------------------------------------
// Arrow up stops the robot rotating
// Arrow down makes the robot rotate anti-clockwise
// Left arrow matches the extracted, subsampled image with the
// selected pattern.

void do_keyboard_stuff() {
 int key = wb_keyboard_get_key();
    while (key > 0) {
      switch (key) {
         case WB_KEYBOARD_UP:
           omegaL = 0;
           omegaR = 0;
           break;
         case WB_KEYBOARD_DOWN:
           omegaL = -0.15*MAX_SPEED;
           omegaR =  0.15*MAX_SPEED;         
           break;
         case WB_KEYBOARD_LEFT:
           printf("\n");
           float sum;
           sum = imageImageOverlap(imageSubs,pattern_image,7,7);
           printf("Overlap sum = %f\n",sum);
           break;
         case WB_KEYBOARD_RIGHT:        
          break;
         case WB_KEYBOARD_PAGEUP:
          break;
         case WB_KEYBOARD_PAGEDOWN:          
          break;
         case (WB_KEYBOARD_SHIFT + WB_KEYBOARD_UP):
           break;                  
      }
      key = wb_keyboard_get_key();
    }
}

// Here's where the execution starts --------------------------
int main() {

  
  int nr_pels; // number of pixels in segmented image
  int tlr,tlc,brr,brc; // top-left row, top-left column etc.
  
  wb_robot_init();

  init();
  
  omegaL = -0.15*MAX_SPEED;
  omegaR =  0.15*MAX_SPEED;
 
  wb_robot_step(TIME_STEP);
  
  // Assign the images and their associated displays in the world
  // Segmented image ----------------------------------------------
  imageSeg = (unsigned char *) malloc(sizeof(unsigned char)*widthCamera*heightCamera*8);
  displaySeg = wb_robot_get_device("display");
  widthImageSeg = wb_display_get_width(displaySeg); 
  heightImageSeg = wb_display_get_height(displaySeg);

  // Subsampled image ----------------------------------------------
  widthImageSubs  = 7;
  heightImageSubs = 7;
  imageSubs = (unsigned char *) malloc(sizeof(unsigned char)*widthImageSubs*heightImageSubs*8);
  displaySubs = wb_robot_get_device("display3");
  
  // Create a test patter image and test the dump functions.
  pattern_image = (unsigned char *) malloc(sizeof(unsigned char)*widthImageSubs*heightImageSubs*8);
  // Test convert pattern array to image
  convertArrayToImage(pattern_1,pattern_image);
  dumpPattern(pattern_1,7,7);
  printf("------------\n");
  dumpImage(pattern_image,7,7);

  // Here is the main controller while loop that is immortal
  while (wb_robot_step(TIME_STEP) != -1) {
    // grab the image from the camera	  
    const unsigned char *imageCamera = wb_camera_get_image(camera);
    // segment the image and get the number of pixels (not used here)
    // (1)
    

    // display the segmented image
    WbImageRef iref1 = wb_display_image_new(displaySeg, widthCamera, heightCamera, imageSeg, WB_IMAGE_BGRA);
    wb_display_image_paste(displaySeg, iref1, 0, 0, false);
   
    // Code to extract the object from the segmented image.
    // get object bounds ready to create image of object
    // The function call returns arguments preceeded by a &
    // (2)
 
 
    if(DEBUG) printf("Nr pels in object image = %d\n",nr_pels);
    if(DEBUG) printf("Obj width = %d, height = %d\n",widthImageObj,heightImageObj);
    if(DEBUG) printf("tlr = %d,tlc = %d, brr = %d, brc = %d\n",tlr,tlc,brr,brc);
    
    // Only proceed if a segmented image exists - test on width and height
    if(widthImageObj > 0 && heightImageObj >0) {
      // Create new image for the object at new width and height established above
      // Remember to free at end of loop since this allocation occurs at each itn of loop!
      imageObj = (unsigned char *) malloc(sizeof(unsigned char)*widthImageObj*heightImageObj*8);

      // create the image of the object isolated
      // (3)


      // grab the display to display the object
      displayObj = wb_robot_get_device("display2");
      // First clear the display
      wb_display_set_color(displayObj,0);
      wb_display_fill_rectangle(displayObj,0,0,widthCamera,heightCamera);   
      // Now paste the image into the display
      WbImageRef iref2 = wb_display_image_new(displayObj, widthImageObj, heightImageObj, imageObj, WB_IMAGE_BGRA);
      wb_display_image_paste(displayObj, iref2, 0, 0, false);
      // Now create the subsampled image
      // (4)



      // grab the display to display the subsampled image      
      displaySubs = wb_robot_get_device("display3");
      // First clear the display
      wb_display_set_color(displaySubs,0);
      wb_display_fill_rectangle(displaySubs,0,0,widthImageSubs,heightImageSubs);   
      // Now paste the image into the display   
      WbImageRef iref3 = wb_display_image_new(displaySubs, widthImageSubs, heightImageSubs, imageSubs, WB_IMAGE_BGRA);
      wb_display_image_paste(displaySubs, iref3, 0, 0, false);          
      // Free the image allocated in the loop else you will eat all your RAM
      free(imageObj); 
    } else {
      // clear the display if no image is found to remove previous stuff
      wb_display_set_color(displayObj,0);
      wb_display_fill_rectangle(displayObj,0,0,widthCamera,heightCamera);
      wb_display_set_color(displaySubs,0);
      wb_display_fill_rectangle(displaySubs,0,0,widthImageSubs,heightImageSubs);     
    }
    
    wb_motor_set_velocity(left_motor, omegaL);
    wb_motor_set_velocity(right_motor, omegaR);
    
    // Here comes the user interaction
    do_keyboard_stuff();
    
  }
 
  wb_robot_cleanup();

  return 0;
}

