// =============================================================================
//
// CBP_IProc_1.c
//
// CBP 27-12-20 Stable Image Processing functions
//
// =============================================================================

#include <webots/camera.h>

int imageSegment(const unsigned char *image,unsigned char *imageThresh, int width, int height) {
  
  int count = 0;
  double minsat = 1.5;
  double maxsat = 0;
  for (int i = 0; i < width ; i++) {
    for (int j = 0; j < height; j++) {
      double red = wb_camera_image_get_red(image, width, i, j);
      double green = wb_camera_image_get_green(image, width, i, j);
      double blue = wb_camera_image_get_blue(image, width, i, j);  
      
      // Calculate the saturation in the image     
      double min = 255;
      if(red < min)   min = red;
      if(green < min) min = green;
      if(blue < min)  min = blue;
      double intens = red + green + blue + 0.000001;
       
      double sat = (intens - min)/intens;
      if(sat < minsat) minsat = sat;
      if(sat > maxsat) maxsat = sat; 
	  // printf("Min sat = %f, max sat = %f\n",minsat,maxsat);
	   
	  // Find and remove white light pollution 
      red -= min;
      blue -= min;
      green -= min;
      
      // Calc the hue from the RGB.  
      double sum = red + green + blue + 0.000001;
      red = red/sum;
      green = green/sum;
      blue = blue/sum;
      
      double numi=0.5*((red - green)+(red - blue));
      double denom=sqrt((red-green)*(red - green)+ (red-blue)*(green - blue));
      double hue = acos(numi/(denom + 0.0000001))*180/M_PI;
      if(blue > green) hue = 360-hue;   

      // 4 bytes per pixel in the image. Offset 1 is red
      *(imageThresh + i*4 + 0 + j*width*4) = 0;
      *(imageThresh + i*4 + 1 + j*width*4) = 0;
      *(imageThresh + i*4 + 2 + j*width*4) = 0;
      *(imageThresh + i*4 + 3 + j*width*4) = 0;      
    
	  // Segment on blue with above 70% saturation
      if(( hue > 235 && hue < 245) && sat > 0.7){  	  
         *(imageThresh + i*4 + 1 + j*width*4) = 250;
         *(imageThresh + i*4 + 2 + j*width*4) = 250;
         *(imageThresh + i*4 + 3 + j*width*4) = 250;                 
        count++;
      }
    }
        
  }

  return count;
  
}
 