// =============================================================================
//
// CBP_IProc_2.c
//
// CBP 28-12-20 Creation of 'L-Template' images for object recognition
//
// =============================================================================
#include <stdio.h>
#include <stdbool.h>
#define DEBUG false

// Input thresholded image, and returns bounding box of image: tlr, tlc, brr, brc

int detectObjectBounds(const unsigned char *imageIn,unsigned char *imageObj,
                         int inWidth, int inHeight, int *widthObj, int *heightObj,
                         int *topLeftRow, int *topLeftCol, int *botRightRow, int *botRightCol) {
                         
  int col = 0;
  int row = 0;
  int minRow = inHeight+1;
  int minCol = inWidth+1;
  int maxRow = -1;
  int maxCol = -1;
  unsigned char pel;
  int nr_pels = 0;
  
  for(row = 0; row < inHeight; row++) {
    for(col = 0; col < inWidth; col++) {
      pel = *(imageIn + col*4 + 1 + row*inWidth*4);
      if(pel > 0) {
        if(row < minRow) minRow = row;
        if(row > maxRow) maxRow = row;        
        if(col < minCol) minCol = col;
        if(col > maxCol) maxCol = col;  
        nr_pels++;      
      }  
    
    }
  
  } 
  if(DEBUG) printf("minR = %d, maxR = %d, minC = %d, maxC = %d\n",minRow,maxRow,minCol,maxCol);
  *widthObj = maxCol - minCol;
  *heightObj = maxRow - minRow;
  *topLeftRow = minRow;
  *topLeftCol = minCol;
  *botRightRow = maxRow;
  *botRightCol = maxCol;
  
  return nr_pels;                      
                         
}     

// Gets object identified in an image using the tlr, tlc, brr, brc from above function
// Extracts object pixels and writes to output image with (tlr, tlc) = (0, 0)

int writeObjectToImage(const unsigned char *imageIn, unsigned char *imageOut, int inWidth,
                       int topLeftRow, int topLeftCol, int botRightRow, int botRightCol) {
                       
  int outHeight = botRightRow - topLeftRow;
  int outWidth  = botRightCol - topLeftCol;
  int inRow, inCol, outRow, outCol;
  unsigned char inPel;
  int nr_pels = 0;
   
  for(outRow = 0; outRow < outHeight; outRow++) {
    for(outCol = 0; outCol < outWidth; outCol++) {
      inRow = outRow + topLeftRow;
      inCol = outCol + topLeftCol;
      inPel = *(imageIn + inCol*4 + 1 + inRow*inWidth*4);
      *(imageOut + outCol*4 + 1 + outRow*outWidth*4) = inPel;
      *(imageOut + outCol*4 + 2 + outRow*outWidth*4) = inPel;
      *(imageOut + outCol*4 + 3 + outRow*outWidth*4) = inPel;      
      nr_pels++;
    }   
  }                      
  return nr_pels;                  
}

// Subsamples the input image to produce an output image of 7 x 7 pixels.
// Could be generalized to MxM output image.

int createSubSampleImage(unsigned char *imageIn, unsigned char *imageOut, int inWidth, int inHeight) {

  int inRow, inCol, outRow, outCol;
  int outHeight = 7;
  int outWidth = 7;
  float widthFactor  = (float)inWidth/(float)outWidth;
  float heightFactor = (float)inHeight/(float)outHeight;
  unsigned char inPel;
  int nr_pels = 0;

  // Set output image to zero
  for(outRow = 0; outRow < outHeight; outRow++) {
    for(outCol = 0; outCol < outWidth; outCol++) {
      *(imageOut + outCol*4 + 1 + outRow*outWidth*4) = 0;
      *(imageOut + outCol*4 + 2 + outRow*outWidth*4) = 0;
      *(imageOut + outCol*4 + 3 + outRow*outWidth*4) = 0;     
    }
  }  
  
  for(outRow = 0; outRow < outHeight; outRow++) {
    for(outCol = 0; outCol < outWidth; outCol++) {
      // Check the condition for being able to subsample
      if(inHeight >= outHeight && inWidth >= outWidth) {
        inRow = (int)(heightFactor*outRow);
        inCol = (int)(widthFactor*outCol);
        inPel = *(imageIn + inCol*4 + 1 + inRow*inWidth*4);
        *(imageOut + outCol*4 + 1 + outRow*outWidth*4) = inPel;
        *(imageOut + outCol*4 + 2 + outRow*outWidth*4) = inPel;
        *(imageOut + outCol*4 + 3 + outRow*outWidth*4) = inPel;
        nr_pels++;
      }
    }
  }
    
  return nr_pels;
}
  